@@ -61,7 +61,7 @@ cmph_t *chm_new(cmph_config_t *mph, double c)
 	chm_config_data_t *chm = (chm_config_data_t *)mph->data;
 	chm->m = mph->key_source->nkeys;
 	if (c == 0) c = 2.09;
-	chm->n = ceil(c * mph->key_source->nkeys);	
+	chm->n = (cmph_uint32)ceil(c * mph->key_source->nkeys);	
 	DEBUGP("m (edges): %u n (vertices): %u c: %f\n", chm->m, chm->n, c);
 	chm->graph = graph_new(chm->n, chm->m);
 	DEBUGP("Created graph\n");
@@ -204,30 +204,32 @@ int chm_dump(cmph_t *mphf, FILE *fd)
 	cmph_uint32 buflen;
 	cmph_uint32 two = 2; //number of hash functions
 	chm_data_t *data = (chm_data_t *)mphf->data;
+	register size_t nbytes;
+	
 	__cmph_dump(mphf, fd);
 
-	fwrite(&two, sizeof(cmph_uint32), (size_t)1, fd);
+	nbytes = fwrite(&two, sizeof(cmph_uint32), (size_t)1, fd);
 	hash_state_dump(data->hashes[0], &buf, &buflen);
 	DEBUGP("Dumping hash state with %u bytes to disk\n", buflen);
-	fwrite(&buflen, sizeof(cmph_uint32), (size_t)1, fd);
-	fwrite(buf, (size_t)buflen, (size_t)1, fd);
+	nbytes = fwrite(&buflen, sizeof(cmph_uint32), (size_t)1, fd);
+	nbytes = fwrite(buf, (size_t)buflen, (size_t)1, fd);
 	free(buf);
 
 	hash_state_dump(data->hashes[1], &buf, &buflen);
 	DEBUGP("Dumping hash state with %u bytes to disk\n", buflen);
-	fwrite(&buflen, sizeof(cmph_uint32), (size_t)1, fd);
-	fwrite(buf, (size_t)buflen, (size_t)1, fd);
+	nbytes = fwrite(&buflen, sizeof(cmph_uint32), (size_t)1, fd);
+	nbytes = fwrite(buf, (size_t)buflen, (size_t)1, fd);
 	free(buf);
 
-	fwrite(&(data->n), sizeof(cmph_uint32), (size_t)1, fd);
-	fwrite(&(data->m), sizeof(cmph_uint32), (size_t)1, fd);
+	nbytes = fwrite(&(data->n), sizeof(cmph_uint32), (size_t)1, fd);
+	nbytes = fwrite(&(data->m), sizeof(cmph_uint32), (size_t)1, fd);
 	
-	fwrite(data->g, sizeof(cmph_uint32)*data->n, (size_t)1, fd);
-	#ifdef DEBUG
+	nbytes = fwrite(data->g, sizeof(cmph_uint32)*data->n, (size_t)1, fd);
+/*	#ifdef DEBUG
 	fprintf(stderr, "G: ");
 	for (i = 0; i < data->n; ++i) fprintf(stderr, "%u ", data->g[i]);
 	fprintf(stderr, "\n");
-	#endif
+	#endif*/
 	return 1;
 }
 
@@ -238,31 +240,31 @@ void chm_load(FILE *f, cmph_t *mphf)
 	cmph_uint32 buflen;
 	cmph_uint32 i;
 	chm_data_t *chm = (chm_data_t *)malloc(sizeof(chm_data_t));
-
+	register size_t nbytes;
 	DEBUGP("Loading chm mphf\n");
 	mphf->data = chm;
-	fread(&nhashes, sizeof(cmph_uint32), (size_t)1, f);
+	nbytes = fread(&nhashes, sizeof(cmph_uint32), (size_t)1, f);
 	chm->hashes = (hash_state_t **)malloc(sizeof(hash_state_t *)*(nhashes + 1));
 	chm->hashes[nhashes] = NULL;
 	DEBUGP("Reading %u hashes\n", nhashes);
 	for (i = 0; i < nhashes; ++i)
 	{
 		hash_state_t *state = NULL;
-		fread(&buflen, sizeof(cmph_uint32), (size_t)1, f);
+		nbytes = fread(&buflen, sizeof(cmph_uint32), (size_t)1, f);
 		DEBUGP("Hash state has %u bytes\n", buflen);
 		buf = (char *)malloc((size_t)buflen);
-		fread(buf, (size_t)buflen, (size_t)1, f);
+		nbytes = fread(buf, (size_t)buflen, (size_t)1, f);
 		state = hash_state_load(buf, buflen);
 		chm->hashes[i] = state;
 		free(buf);
 	}
 
 	DEBUGP("Reading m and n\n");
-	fread(&(chm->n), sizeof(cmph_uint32), (size_t)1, f);	
-	fread(&(chm->m), sizeof(cmph_uint32), (size_t)1, f);	
+	nbytes = fread(&(chm->n), sizeof(cmph_uint32), (size_t)1, f);	
+	nbytes = fread(&(chm->m), sizeof(cmph_uint32), (size_t)1, f);	
 
 	chm->g = (cmph_uint32 *)malloc(sizeof(cmph_uint32)*chm->n);
-	fread(chm->g, chm->n*sizeof(cmph_uint32), (size_t)1, f);
+	nbytes = fread(chm->g, chm->n*sizeof(cmph_uint32), (size_t)1, f);
 	#ifdef DEBUG
 	fprintf(stderr, "G: ");
 	for (i = 0; i < chm->n; ++i) fprintf(stderr, "%u ", chm->g[i]);
@@ -344,7 +346,7 @@ cmph_uint32 chm_packed_size(cmph_t *mphf)
 	CMPH_HASH h1_type = hash_get_type(data->hashes[0]); 
 	CMPH_HASH h2_type = hash_get_type(data->hashes[1]); 
 
-	return (sizeof(CMPH_ALGO) + hash_state_packed_size(h1_type) + hash_state_packed_size(h2_type) + 
+	return (cmph_uint32)(sizeof(CMPH_ALGO) + hash_state_packed_size(h1_type) + hash_state_packed_size(h2_type) + 
 			4*sizeof(cmph_uint32) + sizeof(cmph_uint32)*data->n);
 }
 
