@@ -7,6 +7,8 @@
 #include "fch.h" /* included -- Fabiano */
 #include "bdz.h" /* included -- Fabiano */
 #include "bdz_ph.h" /* included -- Fabiano */
+#include "chd_ph.h" /* included -- Fabiano */
+#include "chd.h" /* included -- Fabiano */
 
 #include <stdlib.h>
 #include <assert.h>
@@ -14,8 +16,7 @@
 //#define DEBUG
 #include "debug.h"
 
-const char *cmph_names[] = {"bmz", "bmz8", "chm", "brz", "fch", "bdz", 
-							"bdz_ph", NULL }; /* included -- Fabiano */
+const char *cmph_names[] = {"bmz", "bmz8", "chm", "brz", "fch", "bdz", "bdz_ph", "chd_ph", "chd", NULL }; /* included -- Fabiano */
 
 typedef struct 
 {
@@ -74,7 +75,7 @@ static int key_nlfile_read(void *data, char **key, cmph_uint32 *keylen)
 		(*key)[(*keylen) - 1] = 0;
 		--(*keylen);
 	}
-	return *keylen;
+	return (int)(*keylen);
 }
 
 static int key_byte_vector_read(void *data, char **key, cmph_uint32 *keylen)
@@ -87,7 +88,7 @@ static int key_byte_vector_read(void *data, char **key, cmph_uint32 *keylen)
 	*key = (char *)malloc(size);
 	memcpy(*key, keys_vd[cmph_vector->position] + sizeof(*keylen), size);
 	cmph_vector->position = cmph_vector->position + 1;
-	return *keylen;
+	return (int)(*keylen);
 
 }
 
@@ -100,8 +101,8 @@ static int key_struct_vector_read(void *data, char **key, cmph_uint32 *keylen)
     size = *keylen;
     *key = (char *)malloc(size);
     memcpy(*key, (keys_vd + (cmph_struct_vector->position * cmph_struct_vector->struct_size) + cmph_struct_vector->key_offset), size);
-	cmph_struct_vector->position = cmph_struct_vector->position + 1;
-	return *keylen;
+    cmph_struct_vector->position = cmph_struct_vector->position + 1;
+    return (int)(*keylen);
 }
 
 static int key_vector_read(void *data, char **key, cmph_uint32 *keylen)
@@ -109,12 +110,12 @@ static int key_vector_read(void *data, char **key, cmph_uint32 *keylen)
         cmph_vector_t *cmph_vector = (cmph_vector_t *)data;
         char **keys_vd = (char **)cmph_vector->vector;
 	size_t size;
-        *keylen = strlen(keys_vd[cmph_vector->position]);
+        *keylen = (cmph_uint32)strlen(keys_vd[cmph_vector->position]);
 	size = *keylen;
         *key = (char *)malloc(size + 1);
         strcpy(*key, keys_vd[cmph_vector->position]);
         cmph_vector->position = cmph_vector->position + 1;
-	return *keylen;
+	return (int)(*keylen);
 
 }
 
@@ -150,11 +151,12 @@ static void key_vector_rewind(void *data)
 static cmph_uint32 count_nlfile_keys(FILE *fd)
 {
 	cmph_uint32 count = 0;
+	register char * ptr;
 	rewind(fd);
 	while(1)
 	{
 		char buf[BUFSIZ];
-		fgets(buf, BUFSIZ, fd); 
+		ptr = fgets(buf, BUFSIZ, fd); 
 		if (feof(fd)) break;
 		if (buf[strlen(buf) - 1] != '\n') continue;
 		++count;
@@ -321,6 +323,12 @@ void cmph_config_set_algo(cmph_config_t *mph, CMPH_ALGO algo)
 			case CMPH_BDZ_PH:
 				bdz_ph_config_destroy(mph);
 				break;
+			case CMPH_CHD_PH:
+				chd_ph_config_destroy(mph);
+				break;
+			case CMPH_CHD:
+				chd_config_destroy(mph);
+				break;
 			default:
 				assert(0);
 		}
@@ -347,6 +355,12 @@ void cmph_config_set_algo(cmph_config_t *mph, CMPH_ALGO algo)
 			case CMPH_BDZ_PH:
 				mph->data = bdz_ph_config_new();
 				break;
+			case CMPH_CHD_PH:
+				mph->data = chd_ph_config_new();
+				break;
+			case CMPH_CHD:
+				mph->data = chd_config_new(mph);
+				break;
 			default:
 				assert(0);
 		}
@@ -381,6 +395,26 @@ void cmph_config_set_b(cmph_config_t *mph, cmph_uint32 b)
 	{
 		bdz_config_set_b(mph, b);
 	}
+	else if (mph->algo == CMPH_CHD_PH) 
+	{
+		chd_ph_config_set_b(mph, b);
+	}
+	else if (mph->algo == CMPH_CHD) 
+	{
+		chd_config_set_b(mph, b);
+	}
+}
+
+void cmph_config_set_keys_per_bin(cmph_config_t *mph, cmph_uint32 keys_per_bin)
+{
+	if (mph->algo == CMPH_CHD_PH) 
+	{
+		chd_ph_config_set_keys_per_bin(mph, keys_per_bin);
+	}
+	else if (mph->algo == CMPH_CHD) 
+	{
+		chd_config_set_keys_per_bin(mph, keys_per_bin);
+	}
 }
 
 void cmph_config_set_memory_availability(cmph_config_t *mph, cmph_uint32 memory_availability)
@@ -405,19 +439,25 @@ void cmph_config_destroy(cmph_config_t *mph)
 				bmz_config_destroy(mph);
 				break;
 			case CMPH_BMZ8: /* included -- Fabiano */
-	        	bmz8_config_destroy(mph);
+				bmz8_config_destroy(mph);
 				break;
 			case CMPH_BRZ: /* included -- Fabiano */
-	        	brz_config_destroy(mph);
+				brz_config_destroy(mph);
 				break;
 			case CMPH_FCH: /* included -- Fabiano */
-	        	fch_config_destroy(mph);
+				fch_config_destroy(mph);
 				break;
 			case CMPH_BDZ: /* included -- Fabiano */
-	        	bdz_config_destroy(mph);
+				bdz_config_destroy(mph);
 				break;
 			case CMPH_BDZ_PH: /* included -- Fabiano */
-	        	bdz_ph_config_destroy(mph);
+				bdz_ph_config_destroy(mph);
+				break;
+			case CMPH_CHD_PH: /* included -- Fabiano */
+				chd_ph_config_destroy(mph);
+				break;
+			case CMPH_CHD: /* included -- Fabiano */
+				chd_config_destroy(mph);
 				break;
 			default:
 				assert(0);
@@ -456,6 +496,12 @@ void cmph_config_set_hashfuncs(cmph_config_t *mph, CMPH_HASH *hashfuncs)
 		case CMPH_BDZ_PH: /* included -- Fabiano */
 			bdz_ph_config_set_hashfuncs(mph, hashfuncs);
 			break;
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			chd_ph_config_set_hashfuncs(mph, hashfuncs);
+			break;
+		case CMPH_CHD: /* included -- Fabiano */
+			chd_config_set_hashfuncs(mph, hashfuncs);
+			break;
 		default:
 			break;
 	}
@@ -505,6 +551,14 @@ cmph_t *cmph_new(cmph_config_t *mph)
 			DEBUGP("Creating bdz_ph hash\n");
 			mphf = bdz_ph_new(mph, c);
 			break;
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			DEBUGP("Creating chd_ph hash\n");
+			mphf = chd_ph_new(mph, c);
+			break;
+		case CMPH_CHD: /* included -- Fabiano */
+			DEBUGP("Creating chd hash\n");
+			mphf = chd_new(mph, c);
+			break;
 		default:
 			assert(0);
 	}
@@ -518,17 +572,21 @@ int cmph_dump(cmph_t *mphf, FILE *f)
 		case CMPH_CHM:
 			return chm_dump(mphf, f);
 		case CMPH_BMZ: /* included -- Fabiano */
-	        return bmz_dump(mphf, f);
+			return bmz_dump(mphf, f);
 		case CMPH_BMZ8: /* included -- Fabiano */
-	        return bmz8_dump(mphf, f);
+			return bmz8_dump(mphf, f);
 		case CMPH_BRZ: /* included -- Fabiano */
-	        return brz_dump(mphf, f);
+			return brz_dump(mphf, f);
 		case CMPH_FCH: /* included -- Fabiano */
-	        return fch_dump(mphf, f);
+			return fch_dump(mphf, f);
 		case CMPH_BDZ: /* included -- Fabiano */
-	        return bdz_dump(mphf, f);
+			return bdz_dump(mphf, f);
 		case CMPH_BDZ_PH: /* included -- Fabiano */
-	        return bdz_ph_dump(mphf, f);
+			return bdz_ph_dump(mphf, f);
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			return chd_ph_dump(mphf, f);
+		case CMPH_CHD: /* included -- Fabiano */
+			return chd_dump(mphf, f);
 		default:
 			assert(0);
 	}
@@ -572,6 +630,14 @@ cmph_t *cmph_load(FILE *f)
 			DEBUGP("Loading bdz_ph algorithm dependent parts\n");
 			bdz_ph_load(f, mphf);
 			break;
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			DEBUGP("Loading chd_ph algorithm dependent parts\n");
+			chd_ph_load(f, mphf);
+			break;
+		case CMPH_CHD: /* included -- Fabiano */
+			DEBUGP("Loading chd algorithm dependent parts\n");
+			chd_load(f, mphf);
+			break;
 		default:
 			assert(0);
 	}
@@ -605,6 +671,12 @@ cmph_uint32 cmph_search(cmph_t *mphf, const char *key, cmph_uint32 keylen)
 		case CMPH_BDZ_PH: /* included -- Fabiano */
 		        DEBUGP("bdz_ph algorithm search\n");		         
 		        return bdz_ph_search(mphf, key, keylen);
+		case CMPH_CHD_PH: /* included -- Fabiano */
+		        DEBUGP("chd_ph algorithm search\n");		         
+		        return chd_ph_search(mphf, key, keylen);
+		case CMPH_CHD: /* included -- Fabiano */
+		        DEBUGP("chd algorithm search\n");		         
+		        return chd_search(mphf, key, keylen);
 		default:
 			assert(0);
 	}
@@ -625,22 +697,28 @@ void cmph_destroy(cmph_t *mphf)
 			chm_destroy(mphf);
 			return;
 		case CMPH_BMZ: /* included -- Fabiano */
-	        bmz_destroy(mphf);
+			bmz_destroy(mphf);
 			return;
 		case CMPH_BMZ8: /* included -- Fabiano */
-	        bmz8_destroy(mphf);
+			bmz8_destroy(mphf);
 			return;
 		case CMPH_BRZ: /* included -- Fabiano */
-	        brz_destroy(mphf);
+			brz_destroy(mphf);
 			return;
 		case CMPH_FCH: /* included -- Fabiano */
-	        fch_destroy(mphf);
+			fch_destroy(mphf);
 			return;
 		case CMPH_BDZ: /* included -- Fabiano */
-	        bdz_destroy(mphf);
+			bdz_destroy(mphf);
 			return;
 		case CMPH_BDZ_PH: /* included -- Fabiano */
-	    	bdz_ph_destroy(mphf);
+			bdz_ph_destroy(mphf);
+			return;
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			chd_ph_destroy(mphf);
+			return;
+		case CMPH_CHD: /* included -- Fabiano */
+			chd_destroy(mphf);
 			return;
 		default: 
 			assert(0);
@@ -684,6 +762,12 @@ void cmph_pack(cmph_t *mphf, void *packed_mphf)
 		case CMPH_BDZ_PH: /* included -- Fabiano */
 			bdz_ph_pack(mphf, ptr);
 			break;
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			chd_ph_pack(mphf, ptr);
+			break;
+		case CMPH_CHD: /* included -- Fabiano */
+			chd_pack(mphf, ptr);
+			break;
 		default: 
 			assert(0);
 	}
@@ -713,6 +797,10 @@ cmph_uint32 cmph_packed_size(cmph_t *mphf)
 			return bdz_packed_size(mphf);
 		case CMPH_BDZ_PH: /* included -- Fabiano */
 			return bdz_ph_packed_size(mphf);
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			return chd_ph_packed_size(mphf);
+		case CMPH_CHD: /* included -- Fabiano */
+			return chd_packed_size(mphf);
 		default: 
 			assert(0);
 	}
@@ -746,6 +834,10 @@ cmph_uint32 cmph_search_packed(void *packed_mphf, const char *key, cmph_uint32 k
 			return bdz_search_packed(++ptr, key, keylen);
 		case CMPH_BDZ_PH: /* included -- Fabiano */
 			return bdz_ph_search_packed(++ptr, key, keylen);
+		case CMPH_CHD_PH: /* included -- Fabiano */
+			return chd_ph_search_packed(++ptr, key, keylen);
+		case CMPH_CHD: /* included -- Fabiano */
+			return chd_search_packed(++ptr, key, keylen);
 		default: 
 			assert(0);
 	}
